// CodeMirror, copyright (c) by Marijn Haverbeke and others
// Distributed under an MIT license: https://codemirror.net/5/LICENSE

; (function (mod) {
  if (typeof exports == 'object' && typeof module == 'object')
    // CommonJS
    mod(require('../../lib/codemirror'))
  else if (typeof define == 'function' && define.amd)
    // AMD
    define(['../../lib/codemirror'], mod)
  // Plain browser env
  else mod(CodeMirror)
})(function (CodeMirror) {
  'use strict'

  CodeMirror.registerGlobalHelper(
    'fold',
    'comment',
    function (mode) {
      return mode.blockCommentStart && mode.blockCommentEnd
    },
    function (cm, start) {
      const mode = cm.getModeAt(start);
      const startToken = mode.blockCommentStart;
      const endToken = mode.blockCommentEnd;

      // Return early if no block comment markers are found
      if (!startToken || !endToken) return;

      const line = start.line;
      const lineText = cm.getLine(line);

      function findCommentStart() {
        let at = start.ch;
        let pass = 0;

        while (true) {
          let found = at <= 0 ? -1 : lineText.lastIndexOf(startToken, at - 1);
          if (found === -1) {
            if (pass === 1) return null;
            pass = 1;
            at = lineText.length;
            continue;
          }
          if (pass === 1 && found < start.ch) return null;

          const tokenType = cm.getTokenTypeAt(CodeMirror.Pos(line, found + 1));
          if (/comment/.test(tokenType) &&
            (found === 0 ||
              lineText.slice(found - endToken.length, found) === endToken ||
              !/comment/.test(cm.getTokenTypeAt(CodeMirror.Pos(line, found))))) {
            return found + startToken.length;
          }

          at = found - 1;
        }
      }

      const startCh = findCommentStart();
      if (startCh === null) return;

      function findCommentEnd() {
        let depth = 1;
        const lastLine = cm.lastLine();
        let end;
        let endCh;

        // A helper function to process each line for finding the comment end
        function processLine(i, pos) {
          const text = cm.getLine(i);

          while (pos < text.length) {
            const nextOpen = text.indexOf(startToken, pos);
            const nextClose = text.indexOf(endToken, pos);

            pos = Math.min(nextOpen < 0 ? text.length : nextOpen, nextClose < 0 ? text.length : nextClose);
            if (pos === text.length) break;

            if (pos === nextOpen) {
              depth++;
            } else if (--depth === 0) {
              end = i;
              endCh = pos;
              return true;  // Return true if the comment block is closed
            }
            ++pos;
          }
          return false;  // Continue searching in this line
        }

        // Iterate over lines and process them
        for (let i = line; i <= lastLine; ++i) {
          let pos = (i === line) ? startCh : 0;
          if (processLine(i, pos)) {
            return { end, endCh };
          }
        }

        return null;  // If no matching end is found
      }

      const endInfo = findCommentEnd();
      if (endInfo === null) return;

      return {
        from: CodeMirror.Pos(line, startCh),
        to: CodeMirror.Pos(endInfo.end, endInfo.endCh),
      };
    }


  )
})
