// CodeMirror, copyright (c) by Marijn Haverbeke and others
// Distributed under an MIT license: https://codemirror.net/5/LICENSE

; (function (mod) {
  if (typeof exports == 'object' && typeof module == 'object')
    // CommonJS
    mod(require('../../lib/codemirror'))
  else if (typeof define == 'function' && define.amd)
    // AMD
    define(['../../lib/codemirror'], mod)
  // Plain browser env
  else mod(CodeMirror)
})(function (CodeMirror) {
  'use strict'

  CodeMirror.registerHelper('fold', 'markdown', function (cm, start) {
    const maxDepth = 100;

    function isHeader(lineNo) {
      const tokentype = cm.getTokenTypeAt(CodeMirror.Pos(lineNo, 0));
      return tokentype && /\bheader\b/.test(tokentype);
    }

    function headerLevel(lineNo, line, nextLine) {
      const match = line && line.match(/^#+/);
      if (match && isHeader(lineNo)) return match[0].length;
      const nextMatch = nextLine && nextLine.match(/^[=\-]+\s*$/);
      if (nextMatch && isHeader(lineNo + 1)) return nextLine[0] === '=' ? 1 : 2;
      return maxDepth;
    }

    // Extract first line and the next line
    const firstLine = cm.getLine(start.line);
    let nextLine = cm.getLine(start.line + 1);
    const level = headerLevel(start.line, firstLine, nextLine);
    if (level === maxDepth) return undefined;

    let end = start.line;
    let nextNextLine = cm.getLine(end + 2);
    const lastLineNo = cm.lastLine();

    // Simplify the loop by using an early return
    while (end < lastLineNo && headerLevel(end + 1, nextLine, nextNextLine) > level) {
      ++end;
      nextLine = nextNextLine;
      nextNextLine = cm.getLine(end + 2);
    }

    return {
      from: CodeMirror.Pos(start.line, firstLine.length),
      to: CodeMirror.Pos(end, cm.getLine(end).length),
    };
  });

})
